@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2003, 2004, 2005 Free Software Foundation
+ * Copyright (C) 2003, 2004, 2005, 2007, 2008 Free Software Foundation
  *
- * Author: Nikos Mavroyanopoulos
+ * Author: Nikos Mavrogiannopoulos
  *
  * This file is part of GNUTLS.
  *
@@ -30,8 +30,7 @@
 #include <gnutls_global.h>
 #include <libtasn1.h>
 #include <common.h>
-#include <x509.h>
-#include <extensions.h>
+#include <x509_int.h>
 #include <gnutls_datum.h>
 
 /* This function will attempt to return the requested extension found in
@@ -492,15 +491,15 @@ _gnutls_x509_ext_extract_keyUsage (uint16_t * keyUsage,
 /* extract the basicConstraints from the DER encoded extension
  */
 int
-_gnutls_x509_ext_extract_basicConstraints (int *CA, opaque * extnValue,
+_gnutls_x509_ext_extract_basicConstraints (int *CA,
+					   int *pathLenConstraint,
+					   opaque * extnValue,
 					   int extnValueLen)
 {
   ASN1_TYPE ext = ASN1_TYPE_EMPTY;
   char str[128];
   int len, result;
 
-  *CA = 0;
-
   if ((result = asn1_create_element
        (_gnutls_get_pkix (), "PKIX1.BasicConstraints", &ext)) != ASN1_SUCCESS)
     {
@@ -509,7 +508,6 @@ _gnutls_x509_ext_extract_basicConstraints (int *CA, opaque * extnValue,
     }
 
   result = asn1_der_decoding (&ext, extnValue, extnValueLen, NULL);
-
   if (result != ASN1_SUCCESS)
     {
       gnutls_assert ();
@@ -517,32 +515,43 @@ _gnutls_x509_ext_extract_basicConstraints (int *CA, opaque * extnValue,
       return _gnutls_asn2err (result);
     }
 
-  len = sizeof (str) - 1;
-  /* the default value of cA is false.
-   */
-  result = asn1_read_value (ext, "cA", str, &len);
-  if (result != ASN1_SUCCESS)
+  if (pathLenConstraint)
     {
-      asn1_delete_structure (&ext);
-      return 0;
+      result = _gnutls_x509_read_uint (ext, "pathLenConstraint",
+				       pathLenConstraint);
+      if (result == GNUTLS_E_ASN1_ELEMENT_NOT_FOUND)
+	*pathLenConstraint = -1;
+      else if (result != GNUTLS_E_SUCCESS)
+	{
+	  gnutls_assert();
+	  asn1_delete_structure (&ext);
+	  return _gnutls_asn2err (result);
+	}
     }
 
-  asn1_delete_structure (&ext);
-
-  if (strcmp (str, "TRUE") == 0)
+  /* the default value of cA is false.
+   */
+  len = sizeof (str) - 1;
+  result = asn1_read_value (ext, "cA", str, &len);
+  if (result == ASN1_SUCCESS && strcmp (str, "TRUE") == 0)
     *CA = 1;
   else
     *CA = 0;
 
+  asn1_delete_structure (&ext);
 
   return 0;
 }
 
 /* generate the basicConstraints in a DER encoded extension
  * Use 0 or 1 (TRUE) for CA.
+ * Use negative values for pathLenConstraint to indicate that the field
+ * should not be present, >= 0 to indicate set values.
  */
 int
-_gnutls_x509_ext_gen_basicConstraints (int CA, gnutls_datum_t * der_ext)
+_gnutls_x509_ext_gen_basicConstraints (int CA,
+				       int pathLenConstraint,
+				       gnutls_datum_t * der_ext)
 {
   ASN1_TYPE ext = ASN1_TYPE_EMPTY;
   const char *str;
@@ -569,7 +578,21 @@ _gnutls_x509_ext_gen_basicConstraints (int CA, gnutls_datum_t * der_ext)
       return _gnutls_asn2err (result);
     }
 
-  asn1_write_value (ext, "pathLenConstraint", NULL, 0);
+  if (pathLenConstraint < 0)
+    {
+      result = asn1_write_value (ext, "pathLenConstraint", NULL, 0);
+      if (result < 0)
+	result = _gnutls_asn2err (result);
+    }
+  else
+    result = _gnutls_x509_write_uint32 (ext, "pathLenConstraint",
+					pathLenConstraint);
+  if (result < 0)
+    {
+      gnutls_assert ();
+      asn1_delete_structure (&ext);
+      return result;
+    }
 
   result = _gnutls_x509_der_encode (ext, "", der_ext, 0);
 
@@ -921,3 +944,151 @@ cleanup:
 
   return result;
 }
+
+/* extract the proxyCertInfo from the DER encoded extension
+ */
+int
+_gnutls_x509_ext_extract_proxyCertInfo (int *pathLenConstraint,
+					char **policyLanguage,
+					char **policy,
+					size_t *sizeof_policy,
+					opaque * extnValue,
+					int extnValueLen)
+{
+  ASN1_TYPE ext = ASN1_TYPE_EMPTY;
+  int result;
+  gnutls_datum_t value;
+
+  if ((result = asn1_create_element
+       (_gnutls_get_pkix (), "PKIX1.ProxyCertInfo", &ext)) != ASN1_SUCCESS)
+    {
+      gnutls_assert ();
+      return _gnutls_asn2err (result);
+    }
+
+  result = asn1_der_decoding (&ext, extnValue, extnValueLen, NULL);
+  if (result != ASN1_SUCCESS)
+    {
+      gnutls_assert ();
+      asn1_delete_structure (&ext);
+      return _gnutls_asn2err (result);
+    }
+
+  if (pathLenConstraint)
+    {
+      result = _gnutls_x509_read_uint (ext, "pCPathLenConstraint",
+				       pathLenConstraint);
+      if (result == GNUTLS_E_ASN1_ELEMENT_NOT_FOUND)
+	*pathLenConstraint = -1;
+      else if (result != GNUTLS_E_SUCCESS)
+	{
+	  asn1_delete_structure (&ext);
+	  return _gnutls_asn2err (result);
+	}
+    }
+
+  result = _gnutls_x509_read_value (ext, "proxyPolicy.policyLanguage",
+				    &value, 0);
+  if (result < 0)
+    {
+      gnutls_assert ();
+      asn1_delete_structure (&ext);
+      return result;
+    }
+
+  if (policyLanguage)
+    *policyLanguage = gnutls_strdup (value.data);
+
+  result = _gnutls_x509_read_value (ext, "proxyPolicy.policy", &value, 0);
+  if (result == GNUTLS_E_ASN1_ELEMENT_NOT_FOUND)
+    {
+      if (policy)
+	*policy = NULL;
+      if (sizeof_policy)
+	*sizeof_policy = 0;
+    }
+  else if (result < 0)
+    {
+      gnutls_assert ();
+      asn1_delete_structure (&ext);
+      return result;
+    }
+  else
+    {
+      if (policy)
+	*policy = value.data;
+      if (sizeof_policy)
+	*sizeof_policy = value.size;
+    }
+
+  asn1_delete_structure (&ext);
+
+  return 0;
+}
+
+/* generate the proxyCertInfo in a DER encoded extension
+ */
+int
+_gnutls_x509_ext_gen_proxyCertInfo (int pathLenConstraint,
+				    const char *policyLanguage,
+				    const char *policy,
+				    size_t sizeof_policy,
+				    gnutls_datum_t * der_ext)
+{
+  ASN1_TYPE ext = ASN1_TYPE_EMPTY;
+  int result;
+
+  result = asn1_create_element (_gnutls_get_pkix (),
+				"PKIX1.ProxyCertInfo", &ext);
+  if (result != ASN1_SUCCESS)
+    {
+      gnutls_assert ();
+      return _gnutls_asn2err (result);
+    }
+
+  if (pathLenConstraint < 0)
+    {
+      result = asn1_write_value (ext, "pCPathLenConstraint", NULL, 0);
+      if (result < 0)
+	result = _gnutls_asn2err (result);
+    }
+  else
+    result = _gnutls_x509_write_uint32 (ext, "pCPathLenConstraint",
+					pathLenConstraint);
+  if (result < 0)
+    {
+      gnutls_assert ();
+      asn1_delete_structure (&ext);
+      return result;
+    }
+
+  result = asn1_write_value (ext, "proxyPolicy.policyLanguage",
+			     policyLanguage, 1);
+  if (result < 0)
+    {
+      gnutls_assert ();
+      asn1_delete_structure (&ext);
+      return _gnutls_asn2err (result);
+    }
+
+  result = asn1_write_value (ext, "proxyPolicy.policy",
+			     policy, sizeof_policy);
+  if (result < 0)
+    {
+      gnutls_assert ();
+      asn1_delete_structure (&ext);
+      return _gnutls_asn2err (result);
+    }
+
+  result = _gnutls_x509_der_encode (ext, "", der_ext, 0);
+
+  asn1_delete_structure (&ext);
+
+  if (result < 0)
+    {
+      gnutls_assert ();
+      return result;
+    }
+
+  return 0;
+}
