@@ -1,26 +1,25 @@
 /*
- * Copyright (C) 2004, 2006, 2007 Free Software Foundation
+ * Copyright (C) 2004, 2006, 2007, 2008 Free Software Foundation
  * Copyright (C) 2001,2002 Paul Sheer
- * Portions Copyright (C) 2002,2003 Nikos Mavroyanopoulos
+ * Portions Copyright (C) 2002,2003 Nikos Mavrogiannopoulos
  *
  * This file is part of GNUTLS.
  *
- * GNUTLS is free software; you can redistribute it and/or modify
+ * GNUTLS is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
- *
+ *               
  * GNUTLS is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *
+ *                               
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* This server is heavily modified for GNUTLS by Nikos Mavroyanopoulos
+/* This server is heavily modified for GNUTLS by Nikos Mavrogiannopoulos
  * (which means it is quite unreadable)
  */
 
@@ -32,12 +31,16 @@
 #include <sys/types.h>
 #include <string.h>
 #include <gnutls/gnutls.h>
+#include <gcrypt.h>
 #include <gnutls/extra.h>
+#include <gnutls/openpgp.h>
 #include <sys/time.h>
 #include <fcntl.h>
 #include <list.h>
 
 #if defined _WIN32 || defined __WIN32__
+int _win_select(int max_fd, fd_set * rfds, fd_set * wfds, fd_set * efds,
+		const struct timeval *tv);
 #define select _win_select
 #endif
 
@@ -65,7 +68,6 @@ char *psk_passwd;
 char *srp_passwd;
 char *srp_passwd_conf;
 char *pgp_keyring;
-char *pgp_trustdb;
 char *pgp_keyfile;
 char *pgp_certfile;
 char *x509_keyfile;
@@ -337,10 +339,35 @@ static int comp_priority[PRI_MAX];
 static int mac_priority[PRI_MAX];
 static int cert_type_priority[PRI_MAX];
 
+#if ENABLE_OPRFI
+int
+oprfi_callback (gnutls_session_t session,
+		void *userdata,
+		size_t oprfi_len,
+		const unsigned char *in_oprfi,
+		unsigned char *out_oprfi)
+{
+  size_t ourlen = strlen (info.opaque_prf_input);
+  size_t i;
+
+  printf ("- Received Opaque PRF data of %d bytes\n", oprfi_len);
+  printf ("  data: ");
+  for (i = 0; i < oprfi_len; i++)
+    printf ("%02x", in_oprfi[i]);
+  printf ("\n");
+
+  memset(out_oprfi, 0, oprfi_len);
+  strncpy (out_oprfi, info.opaque_prf_input, oprfi_len);
+
+  return 0;
+}
+#endif
+
 gnutls_session_t
 initialize_session (void)
 {
   gnutls_session_t session;
+  const char *err;
 
   gnutls_init (&session, GNUTLS_SERVER);
 
@@ -356,7 +383,11 @@ initialize_session (void)
       gnutls_db_set_ptr (session, NULL);
     }
 
-  gnutls_set_default_priority (session);
+  if (gnutls_priority_set_direct (session, info.priorities, &err) < 0)
+    {
+      fprintf(stderr, "Syntax error at: %s\n", err);
+      exit(1);
+    }
 
   if (cipher_priority[0])
     gnutls_cipher_set_priority (session, cipher_priority);
@@ -391,6 +422,16 @@ initialize_session (void)
       gnutls_certificate_server_set_request (session, GNUTLS_CERT_REQUEST);
   }
 
+  /* Set maximum compatibility mode. This is only suggested on public webservers
+   * that need to trade security for compatibility
+   */
+   gnutls_session_enable_compatibility_mode( session);
+
+#ifdef ENABLE_OPRFI
+  if (info.opaque_prf_input)
+    gnutls_oprfi_enable_server (session, oprfi_callback, NULL);
+#endif
+
   return session;
 }
 
@@ -448,7 +489,7 @@ peer_print_info (gnutls_session_t session, int *ret_length, const char *header)
 	  if (gnutls_x509_crt_init (&cert) == 0 &&
 	      gnutls_x509_crt_import (cert, &cert_list[i],
 				      GNUTLS_X509_FMT_DER) == 0 &&
-	      gnutls_x509_crt_print (cert, GNUTLS_X509_CRT_FULL, &info) == 0)
+	      gnutls_x509_crt_print (cert, GNUTLS_CRT_PRINT_FULL, &info) == 0)
 	    {
 	      const char *post = "</PRE><P><PRE>";
 
@@ -789,6 +830,8 @@ main (int argc, char **argv)
       strcpy (name, "Echo Server");
     }
 
+  gcry_control (GCRYCTL_ENABLE_QUICK_RANDOM, 0);
+
   if ((ret = gnutls_global_init ()) < 0)
     {
       fprintf (stderr, "global_init: %s\n", gnutls_strerror (ret));
@@ -797,12 +840,6 @@ main (int argc, char **argv)
   gnutls_global_set_log_function (tls_log_func);
   gnutls_global_set_log_level (debug);
 
-  if ((ret = gnutls_global_init_extra ()) < 0)
-    {
-      fprintf (stderr, "global_init_extra: %s\n", gnutls_strerror (ret));
-      exit (1);
-    }
-
   /* Note that servers must generate parameters for
    * Diffie Hellman. See gnutls_dh_params_generate(), and
    * gnutls_dh_params_set().
@@ -862,7 +899,7 @@ main (int argc, char **argv)
   if (pgp_keyring != NULL)
     {
       ret =
-	gnutls_certificate_set_openpgp_keyring_file (cert_cred, pgp_keyring);
+	gnutls_certificate_set_openpgp_keyring_file (cert_cred, pgp_keyring, GNUTLS_OPENPGP_FMT_BASE64);
       if (ret < 0)
 	{
 	  fprintf (stderr, "Error setting the OpenPGP keyring file\n");
@@ -870,25 +907,23 @@ main (int argc, char **argv)
 	}
     }
 
-  if (pgp_trustdb != NULL)
+  if (pgp_certfile != NULL)
     {
-      ret = gnutls_certificate_set_openpgp_trustdb (cert_cred, pgp_trustdb);
+      if (info.pgp_subkey != NULL)
+	ret = gnutls_certificate_set_openpgp_key_file2
+	  (cert_cred, pgp_certfile, pgp_keyfile, info.pgp_subkey, GNUTLS_OPENPGP_FMT_BASE64);
+      else
+	ret = gnutls_certificate_set_openpgp_key_file
+	  (cert_cred, pgp_certfile, pgp_keyfile, GNUTLS_OPENPGP_FMT_BASE64);
+
       if (ret < 0)
 	{
-	  fprintf (stderr, "Error setting the OpenPGP trustdb file\n");
+	  fprintf (stderr,
+		   "Error[%d] while reading the OpenPGP key pair ('%s', '%s')\n",
+		   ret, pgp_certfile, pgp_keyfile);
 	  GERR (ret);
 	}
     }
-
-  if (pgp_certfile != NULL)
-    if ((ret = gnutls_certificate_set_openpgp_key_file
-	 (cert_cred, pgp_certfile, pgp_keyfile)) < 0)
-      {
-	fprintf (stderr,
-		 "Error[%d] while reading the OpenPGP key pair ('%s', '%s')\n",
-		 ret, pgp_certfile, pgp_keyfile);
-	GERR (ret);
-      }
 #endif
 
   if (x509_certfile != NULL)
@@ -952,6 +987,17 @@ main (int argc, char **argv)
 	  GERR (ret);
 	}
 
+      if (info.psk_hint)
+	{
+	  ret = gnutls_psk_set_server_credentials_hint (psk_cred,
+							info.psk_hint);
+	  if (ret)
+	    {
+	      fprintf (stderr, "Error setting PSK identity hint.\n");
+	      GERR (ret);
+	    }
+	}
+
       gnutls_psk_set_server_params_function (psk_cred, get_params);
     }
 #endif
@@ -1108,7 +1154,7 @@ main (int argc, char **argv)
 				addr_ntop ((struct sockaddr *)&client_address, calen,
 					   topbuf, sizeof (topbuf)),
 				get_port (&client_address));
-			print_info (j->tls_session, NULL);
+			print_info (j->tls_session, NULL, 1);
 		      }
 		    j->handshake_ok = 1;
 		  }
@@ -1205,7 +1251,7 @@ main (int argc, char **argv)
 					   topbuf, sizeof (topbuf)),
 				get_port (&client_address));
 
-			print_info (j->tls_session, NULL);
+			print_info (j->tls_session, NULL, 1);
 		      }
 		    j->handshake_ok = 1;
 		  }
@@ -1352,7 +1398,6 @@ gaa_parser (int argc, char **argv)
   psk_passwd = info.psk_passwd;
 
   pgp_keyring = info.pgp_keyring;
-  pgp_trustdb = info.pgp_trustdb;
 
   parse_protocols (info.proto, info.nproto, protocol_priority);
   parse_ciphers (info.ciphers, info.nciphers, cipher_priority);
@@ -1484,11 +1529,11 @@ wrap_db_delete (void *dbf, gnutls_datum_t key)
 void
 print_serv_license (void)
 {
-  fputs ("\nCopyright (C) 2001-2003 Paul Sheer, Nikos Mavroyanopoulos\n"
+  fputs ("\nCopyright (C) 2001-2003 Paul Sheer, Nikos Mavrogiannopoulos\n"
 	 "\nCopyright (C) 2004 Free Software Foundation\n"
 	 "This program is free software; you can redistribute it and/or modify \n"
 	 "it under the terms of the GNU General Public License as published by \n"
-	 "the Free Software Foundation; either version 2 of the License, or \n"
+	 "the Free Software Foundation; either version 3 of the License, or \n"
 	 "(at your option) any later version. \n" "\n"
 	 "This program is distributed in the hope that it will be useful, \n"
 	 "but WITHOUT ANY WARRANTY; without even the implied warranty of \n"
